@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.db import models, migrations
 
 
@@ -23,7 +20,11 @@ class Migration(migrations.Migration):
             name='success',
             options={'verbose_name_plural': 'Successful tasks', 'verbose_name': 'Successful task'},
         ),
-        migrations.AlterField(
+        migrations.RemoveField(
+            model_name='task',
+            name='id',
+        ),
+        migrations.AddField(
             model_name='task',
             name='id',
             field=models.CharField(max_length=32, primary_key=True, editable=False, serialize=False),
