@@ -1,14 +1,19 @@
+# Django
 from django import get_version
+from django.core.exceptions import ValidationError
+from django.db import models
 from django.template.defaultfilters import truncatechars
-
 from django.urls import reverse
+from django.utils import timezone
 from django.utils.html import format_html
 from django.utils.translation import gettext_lazy as _
-from django.db import models
-from django.utils import timezone
+
+# External
 from picklefield import PickledObjectField
 from picklefield.fields import dbsafe_decode
 
+# Local
+from django_q.conf import croniter
 from django_q.signing import SignedPackage
 
 
@@ -24,6 +29,7 @@ class Task(models.Model):
     started = models.DateTimeField(editable=False)
     stopped = models.DateTimeField(editable=False)
     success = models.BooleanField(default=True, editable=False)
+    attempt_count = models.IntegerField(default=0)
 
     @staticmethod
     def get_result(task_id):
@@ -131,6 +137,16 @@ class Failure(Task):
         proxy = True
 
 
+# Optional Cron validator
+def validate_cron(value):
+    if not croniter:
+        raise ImportError(_("Please install croniter to enable cron expressions"))
+    try:
+        croniter.expand(value)
+    except ValueError as e:
+        raise ValidationError(e)
+
+
 class Schedule(models.Model):
     name = models.CharField(max_length=100, null=True, blank=True)
     func = models.CharField(max_length=256, help_text="e.g. module.tasks.function")
@@ -152,6 +168,7 @@ class Schedule(models.Model):
     MONTHLY = "M"
     QUARTERLY = "Q"
     YEARLY = "Y"
+    CRON = "C"
     TYPE = (
         (ONCE, _("Once")),
         (MINUTES, _("Minutes")),
@@ -161,6 +178,7 @@ class Schedule(models.Model):
         (MONTHLY, _("Monthly")),
         (QUARTERLY, _("Quarterly")),
         (YEARLY, _("Yearly")),
+        (CRON, _("Cron")),
     )
     schedule_type = models.CharField(
         max_length=1, choices=TYPE, default=TYPE[0][0], verbose_name=_("Schedule Type")
@@ -174,6 +192,13 @@ class Schedule(models.Model):
     next_run = models.DateTimeField(
         verbose_name=_("Next Run"), default=timezone.now, null=True
     )
+    cron = models.CharField(
+        max_length=100,
+        null=True,
+        blank=True,
+        validators=[validate_cron],
+        help_text=_("Cron expression"),
+    )
     task = models.CharField(max_length=100, null=True, editable=False)
 
     def success(self):
