@@ -7,9 +7,8 @@ from django.db import IntegrityError
 from django.utils import timezone
 from multiprocessing import Value
 
-from django_q.brokers import get_broker
-
 # local
+from django_q.brokers import get_broker
 from django_q.conf import Conf, logger
 from django_q.humanhash import uuid
 from django_q.models import Schedule, Task
@@ -89,6 +88,7 @@ def schedule(func, *args, **kwargs):
     :param repeats: how many times to repeat. 0=never, -1=always.
     :param next_run: Next scheduled run.
     :type next_run: datetime.datetime
+    :param cron: optional cron expression
     :param kwargs: function keyword arguments.
     :return: the schedule object.
     :rtype: Schedule
@@ -99,13 +99,14 @@ def schedule(func, *args, **kwargs):
     minutes = kwargs.pop("minutes", None)
     repeats = kwargs.pop("repeats", -1)
     next_run = kwargs.pop("next_run", timezone.now())
+    cron = kwargs.pop("cron", None)
 
     # check for name duplicates instead of am unique constraint
     if name and Schedule.objects.filter(name=name).exists():
         raise IntegrityError("A schedule with the same name already exists.")
 
     # create and return the schedule
-    return Schedule.objects.create(
+    s = Schedule(
         name=name,
         func=func,
         hook=hook,
@@ -115,7 +116,12 @@ def schedule(func, *args, **kwargs):
         minutes=minutes,
         repeats=repeats,
         next_run=next_run,
+        cron=cron,
     )
+    # make sure we trigger validation
+    s.full_clean()
+    s.save()
+    return s
 
 
 def result(task_id, wait=0, cached=Conf.CACHED):
@@ -480,7 +486,7 @@ def async_chain(chain, group=None, cached=Conf.CACHED, sync=Conf.SYNC, broker=No
     return group
 
 
-class Iter(object):
+class Iter:
     """
     An async task with iterable arguments
     """
@@ -550,7 +556,7 @@ class Iter(object):
         return len(self.args)
 
 
-class Chain(object):
+class Chain:
     """
     A sequential chain of tasks
     """
@@ -634,7 +640,7 @@ class Chain(object):
         return len(self.chain)
 
 
-class AsyncTask(object):
+class AsyncTask:
     """
     an async task
     """
