@@ -22,7 +22,7 @@ from django_q.tests.tasks import multiply, TaskError
 from django_q.queues import Queue
 
 
-class WordClass(object):
+class WordClass:
     def __init__(self):
         self.word_list = DEFAULT_WORDLIST
 
@@ -45,6 +45,7 @@ def test_sync(broker):
     task = async_task('django_q.tests.tasks.count_letters', DEFAULT_WORDLIST, broker=broker, sync=True)
     assert result(task) == 1506
 
+
 @pytest.mark.django_db
 def test_sync_raise_exception(broker):
     with pytest.raises(TaskError):
@@ -141,6 +142,8 @@ def test_enqueue(broker, admin_user):
     # q_options and save opt_out test
     k = async_task('django_q.tests.tasks.get_user_id', admin_user,
                    q_options={'broker': broker, 'group': 'test_k', 'save': False, 'timeout': 90})
+    # test unicode
+    assert Task(name='Amalia').__unicode__()=='Amalia'
     # check if everything has a task id
     assert isinstance(a, str)
     assert isinstance(b, str)
@@ -335,6 +338,40 @@ def test_recycle(broker, monkeypatch):
     assert Success.objects.count() == Conf.SAVE_LIMIT
     broker.delete_queue()
 
+@pytest.mark.django_db
+def test_max_rss(broker, monkeypatch):
+    # set up the Sentinel
+    broker.list_key = 'test_max_rss_test:q'
+    async_task('django_q.tests.tasks.multiply', 2, 2, broker=broker)
+    start_event = Event()
+    stop_event = Event()
+    cluster_id = uuidlib.uuid4()
+    # override settings
+    monkeypatch.setattr(Conf, 'MAX_RSS', 40000)
+    monkeypatch.setattr(Conf, 'WORKERS', 1)
+    # set a timer to stop the Sentinel
+    threading.Timer(3, stop_event.set).start()
+    s = Sentinel(stop_event, start_event, cluster_id=cluster_id, broker=broker)
+    assert start_event.is_set()
+    assert s.status() == Conf.STOPPED
+    assert s.reincarnations == 1
+    async_task('django_q.tests.tasks.multiply', 2, 2, broker=broker)
+    task_queue = Queue()
+    result_queue = Queue()
+    # push the task
+    pusher(task_queue, stop_event, broker=broker)
+    # worker should exit on recycle
+    worker(task_queue, result_queue, Value('f', -1))
+    # check if the work has been done
+    assert result_queue.qsize() == 1
+    # save_limit test
+    monkeypatch.setattr(Conf, 'SAVE_LIMIT', 1)
+    result_queue.put('STOP')
+    # run monitor
+    monitor(result_queue)
+    assert Success.objects.count() == Conf.SAVE_LIMIT
+    broker.delete_queue()
+
 
 @pytest.mark.django_db
 def test_bad_secret(broker, monkeypatch):
@@ -360,6 +397,41 @@ def test_bad_secret(broker, monkeypatch):
     broker.delete_queue()
 
 
+@pytest.mark.django_db
+def test_attempt_count(broker, monkeypatch):
+    monkeypatch.setattr(Conf, 'MAX_ATTEMPTS', 3)
+    tag = uuid()
+    task = {'id': tag[1],
+            'name': tag[0],
+            'func': 'math.copysign',
+            'args': (1, -1),
+            'kwargs': {},
+            'started': timezone.now(),
+            'stopped': timezone.now(),
+            'success': False,
+            'result': None}
+    # initial save - no success
+    save_task(task, broker)
+    assert Task.objects.filter(id=task['id']).exists()
+    saved_task = Task.objects.get(id=task['id'])
+    assert saved_task.attempt_count == 1
+    sleep(0.5)
+    # second save
+    old_stopped = task['stopped']
+    task['stopped'] = timezone.now()
+    save_task(task, broker)
+    saved_task = Task.objects.get(id=task['id'])
+    assert saved_task.attempt_count == 2
+    # third save -
+    task['stopped'] = timezone.now()
+    save_task(task, broker)
+    saved_task = Task.objects.get(id=task['id'])
+    assert saved_task.attempt_count == 3
+    # task should be removed from queue
+    assert broker.queue_size() == 0
+
+
+
 @pytest.mark.django_db
 def test_update_failed(broker):
     tag = uuid()
@@ -401,6 +473,7 @@ def test_update_failed(broker):
     assert saved_task.success is True
     assert saved_task.result == 'result'
 
+
 @pytest.mark.django_db
 def test_acknowledge_failure_override():
     class VerifyAckMockBroker(Broker):
@@ -434,10 +507,12 @@ def test_acknowledge_failure_override():
 
     tag = uuid()
     task_success_ack = task_fail_ack.copy()
-    task_success_ack.update({'id': tag[1],
-                             'name': tag[0],
-                             'ack_id': 'test_success_ack_id',
-                             'success': True,})
+    task_success_ack.update({
+        'id': tag[1],
+        'name': tag[0],
+        'ack_id': 'test_success_ack_id',
+        'success': True,
+    })
     del task_success_ack['ack_failure']
 
     result_queue = Queue()
@@ -453,6 +528,7 @@ def test_acknowledge_failure_override():
     assert broker.acknowledgements.get('test_fail_no_ack_id') is None
     assert broker.acknowledgements.get('test_success_ack_id') == 1
 
+
 @pytest.mark.django_db
 def assert_result(task):
     assert task is not None
