@@ -3,14 +3,15 @@ from multiprocessing import Event, Value
 
 import arrow
 import pytest
+from django.core.exceptions import ValidationError
 from django.db import IntegrityError
 from django.utils import timezone
 
 from django_q.brokers import get_broker
 from django_q.cluster import pusher, worker, monitor, scheduler
 from django_q.conf import Conf
-from django_q.tasks import Schedule, fetch, schedule as create_schedule
 from django_q.queues import Queue
+from django_q.tasks import Schedule, fetch, schedule as create_schedule
 
 
 @pytest.fixture
@@ -91,8 +92,25 @@ def test_scheduler(broker, monkeypatch):
                                       schedule_type=Schedule.MINUTES,
                                       minutes=10)
     assert hasattr(minute_schedule, 'pk') is True
+    # Cron schedule
+    cron_schedule = create_schedule('django_q.tests.tasks.word_multiply',
+                                    2,
+                                    word='django',
+                                    schedule_type=Schedule.CRON,
+                                    cron="0 22 * * 1-5")
+    assert hasattr(cron_schedule, 'pk') is True
+    assert cron_schedule.full_clean() is None
+    assert cron_schedule.__unicode__() == 'django_q.tests.tasks.word_multiply'
+    with pytest.raises(ValidationError):
+        create_schedule('django_q.tests.tasks.word_multiply',
+                        2,
+                        word='django',
+                        schedule_type=Schedule.CRON,
+                        cron="0 22 * * 1-12")
     # All other types
     for t in Schedule.TYPE:
+        if t[0] == Schedule.CRON:
+            continue
         schedule = create_schedule('django_q.tests.tasks.word_multiply',
                                    2,
                                    word='django',
